const {exec, execSync} = require('child_process');
const argv = require('yargs').argv;
const { existsSync, mkdirSync, writeFileSync } = require('fs');

//Version 0.3.3 should work for all distributions. This code will be handy if some distributions will behave differently
//const distribution = execSync('cat /etc/*release | grep -E ^NAME');
//distribution.includes('Red Hat') ? '0.3.5' :'0.3.3';
const NFS4_ACL_TOOLS_VERSION = '0.3.3';
//const NFS4_ACL_MODULE_NAME = JSON.parse(readFileSync('package.json')).name;

async function execute(command, cwd) {
    console.log('Executing command:', {command, cwd});
    return new Promise((resolve, reject) => {
        const childProcess = exec(command, {cwd}, (error, stdout, stderr) => {
            if (error) {
                reject(stderr || stdout);
            } else {
                resolve(stdout.trim());
            }
        });
        childProcess.stdout.on('data',(data) => console.log(data));
        childProcess.stderr.on('data',(data) => console.error(data));
    });
}

async function preinstall() {
    try {
        console.log('Installing development tools...');
        await execute('yum install gcc-c++ -y; yum groupinstall "Development Tools" -y; yum install yum-utils -y');

        console.log('Downloading sources...');
        await execute(`yumdownloader --source nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}`);

        // We use * because we don't know which distribution of the rpm we will get (amazon, centos, redhat, etc)
        console.log('Extracting sources...');
        await execute(`rpm --define \"_topdir /root/rpmbuild\" -ivh ./nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}*`);

        console.log('Installing RPM dependencies...');
        await execute(`yum-builddep nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION} -y`);

        console.log('Building spec...');
        await execute('rpmbuild -bp nfs4-acl-tools.spec', '/root/rpmbuild/SPECS');

        console.log('Copying nfs-acl-tools to node_modules...');
        await execute(`mv /root/rpmbuild/BUILD/nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}/ .`);

        console.log('Config and make nfs4-acl-tools');
        await execute('./configure; make; make install', `nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}`);

    } catch (error) {
        console.log('Pre Install failed:', error);
    }
}

async function postinstall() {
    try {
        console.log('Installing node-gyp...');
        await execute('npm install -g node-gyp');

        console.log('Copying source files...');
        await execute(`mv libnfs4acl/* nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}/libnfs4acl/`);
        await execute(`mv napi/* nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}/`);

        console.log('Building nfs-acl-tools folder...');
        await execute('node-gyp configure build', `nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}`);
    } catch (error) {
        console.log('Post install failed:', error);
    }
}

const distribution = process.platform === "win32" ? undefined : execSync('grep -Eoi \'red hat|rhel|centos\' /etc/os-release |  head -n +1 | awk \'{print tolower($0)}\'').toString().trim();
if (['red hat','rhel', 'centos'].includes(distribution)) {
    switch (argv.step) {
        case 'preinstall':
            return preinstall();
        case 'postinstall':
            return postinstall();
    }
} else { //To avoid require errors on unsupported distributions
    const packageDir = `nfs4-acl-tools-${NFS4_ACL_TOOLS_VERSION}`;
    if (!existsSync(`${packageDir}/nfs4-acl.js`)) {
        mkdirSync(packageDir);
        writeFileSync(`${packageDir}/nfs4-acl.js`, '')
    }
}

