#include "nfs4_remove_string_aces.c"
#include "nfs4_remove_file_aces.c"
#include "acl_nfs4_xattr_load.c"
#include "nfs4_acl_for_path.c"
#include "acl_nfs4_xattr_size.c"
#include "acl_nfs4_xattr_pack.c"
#include "nfs4_set_acl.c"
#include "nfs4_get_ace_access.c"
#include "nfs4_get_ace_flags.c"
#include "nfs4_get_ace_type.c"
#include "nfs4_print_ace.c"
#include "acl_nfs4_get_whotype.c"
#include "acl_nfs4_get_who.c"
#include "acl_nfs4_set_who.c"
#include "nfs4_new_ace.c"
#include "nfs4_acl_utils.c"
#include "nfs4_ace_from_string.c"
#include "nfs4_print_acl.c"
#include "nfs4_insert_string_aces.c"
#include "nfs4_new_acl.c"
#include "nfs4_free_acl.c"
#include <node_api.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <libgen.h>
#include "libacl_nfs4.h"
#include <sys/types.h>
#include <attr/xattr.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <linux/limits.h>

napi_value setAcl(napi_env env, napi_callback_info info) {
  char path[PATH_MAX];
  char aces[NFS4_MAX_ACLSIZE];
  u32 is_dir;
  napi_status status;
  size_t argc = 3;
  size_t writtenBytes;
  napi_value argv[3];

  status = napi_get_cb_info(env, info, &argc, argv, NULL, NULL);

  if (status != napi_ok) {
    napi_throw_error(env, NULL, "Failed to parse arguments");
  }
  /* Getting path as input */
  status = napi_get_value_string_utf8(env, argv[0], path, PATH_MAX, &writtenBytes);

  if (status != napi_ok) {
    napi_throw_error(env, NULL, "Invalid string was passed as argument");
  }
  /* Getting acl as input */
  status = napi_get_value_string_utf8(env, argv[1], aces, NFS4_MAX_ACLSIZE, &writtenBytes);

  if (status != napi_ok) {
    napi_throw_error(env, NULL, "Invalid string was passed as argument");
  }
  /* Getting is_dir as input */
  status = napi_get_value_uint32(env, argv[2], &is_dir);

  if (status != napi_ok) {
    napi_throw_error(env, NULL, "Invalid uint32 was passed as argument");
  }

  struct nfs4_acl * new_acl;
  new_acl = nfs4_new_acl(is_dir);
  
  nfs4_insert_string_aces(new_acl, aces, 0);
  int res = nfs4_set_acl(new_acl, path);
  nfs4_free_acl(new_acl);
  if (res < 0) {
    char errorMsg[1052] = "Could not set acl on path: ";
    napi_throw_error(env, NULL, strcat(errorMsg, path));	
  }
}

napi_value Init(napi_env env, napi_value exports) {
  napi_status status;
  napi_value fn;

  status = napi_create_function(env, NULL, 0, setAcl, NULL, &fn);
  if (status != napi_ok) {
    napi_throw_error(env, NULL, "Unable to wrap native function");
  }

  status = napi_set_named_property(env, exports, "set", fn);
  if (status != napi_ok) {
    napi_throw_error(env, NULL, "Unable to populate exports");
  }

  return exports;
}


NAPI_MODULE(NODE_GYP_MODULE_NAME, Init)
